==> idkit-0.3.2.tar.gz: lib/src/function/image/func_image.dart <==
import 'dart:async';
import 'dart:math';
import 'dart:typed_data';
import 'dart:ui';
import 'package:idkit/idkit.dart';

/// Some Convenience Handling of Fultter Images.
extension IImage on Image {
  /// Image size.
  Size get size => Size(width * 1, height * 1);

  /// Image rectangle with coordinates at (0,0) point.
  Rect get rect => Offset.zero & size;

  /// Get the Uint8List data of the image.
  Future<Uint8List?> uint8List(
      {ImageByteFormat format = ImageByteFormat.rawRgba}) async {
    final ByteData? byteData = await toByteData(format: format);
    return byteData?.buffer.asUint8List();
  }

  /// Get the Uint8List data of the image.
  Future<Uint32List?> uint32List(
      {ImageByteFormat format = ImageByteFormat.rawRgba}) async {
    final ByteData? byteData = await toByteData(format: format);
    return byteData?.buffer.asUint32List();
  }

  /// Change image size.
  Image changeSizeSync({
    int? imgW,
    int? imgH,
    bool isDispose = false,
  }) {
    final int imgW0 = imgW ?? width;
    final int imgH0 = imgH ?? height;
    final Rect rect = Offset.zero & Size(imgW0 * 1, imgH0 * 1);
    final PictureRecorder pictureRecorder = PictureRecorder();
    final Canvas canvas = Canvas(pictureRecorder, rect);
    canvas.drawImageRect(this, Offset.zero & size, rect, Paint());
    final Picture picture = pictureRecorder.endRecording();
    if (isDispose) dispose();
    return picture.toImageSync(imgW0, imgH0);
  }

  /// Resize an image asynchronously.
  Future<Image> changeSize({
    int? imgW,
    int? imgH,
    bool isDispose = false,
  }) {
    final int imgW0 = imgW ?? width;
    final int imgH0 = imgH ?? height;
    final Rect rect = Offset.zero & Size(imgW0 * 1, imgH0 * 1);
    final PictureRecorder pictureRecorder = PictureRecorder();
    final Canvas canvas = Canvas(pictureRecorder, rect);
    canvas.drawImageRect(this, Offset.zero & size, rect, Paint());
    final Picture picture = pictureRecorder.endRecording();
    if (isDispose) dispose();
    return picture.toImage(imgW0, imgH0);
  }

  /// Image scaling.
  Image scaleSync({
    double scale = 1,
    bool isDispose = false,
  }) {
    final int imgW0 = (width * scale).toInt();
    final int imgH0 = (height * scale).toInt();
    final Rect rect = Offset.zero & Size(imgW0 * 1, imgH0 * 1);
    final PictureRecorder pictureRecorder = PictureRecorder();
    final Canvas canvas = Canvas(pictureRecorder, rect);
    canvas.drawImageRect(this, Offset.zero & size, rect, Paint());
    final Picture picture = pictureRecorder.endRecording();
    if (isDispose) dispose();
    return picture.toImageSync(imgW0, imgH0);
  }

  /// Image scaling.
  Future<Image> scale({
    double scale = 1,
    bool isDispose = false,
  }) {
    final int imgW0 = (width * scale).toInt();
    final int imgH0 = (height * scale).toInt();
    final Rect rect = Offset.zero & Size(imgW0 * 1, imgH0 * 1);
    final PictureRecorder pictureRecorder = PictureRecorder();
    final Canvas canvas = Canvas(pictureRecorder, rect);
    canvas.drawImageRect(this, Offset.zero & size, rect, Paint());
    final Picture picture = pictureRecorder.endRecording();
    if (isDispose) dispose();
    return picture.toImage(imgW0, imgH0);
  }

  /// Image decolorization.
  Future<Image?> removalColor({
    Color color = const Color(0xffffffff),
    double tolerance = 1,
  }) async {
    Completer<Image?> completer = Completer();
    final ByteData? byteData =
        await toByteData(format: ImageByteFormat.rawRgba);
    final double cL =
        (0.299 * color.red + 0.587 * color.green + 0.114 * color.blue) / 255;
    final double cA = (color.green - color.red) / 255.0;
    final double cB = (color.blue - (color.green + color.red) / 2) / 255.0;
    if (byteData.isNoNull) {
      final Uint32List uint32list = byteData!.buffer.asUint32List();
      uint32list.iFor((index, element) {
        final Color srcColor = Color(element);
        final double sL = (0.299 * srcColor.red +
                0.587 * srcColor.green +
                0.114 * srcColor.blue) /
            255;
        final double sA = (srcColor.green - srcColor.red) / 255.0;
        final double sB =
            (srcColor.blue - (srcColor.green + srcColor.red) / 2) / 255.0;
        final double dL = cL - sL;
        final double dA = cA - sA;
        final double dB = cB - sB;
        final double dE = sqrt(pow(dL, 2) + pow(dA, 2) + pow(dB, 2));
        if (dE < tolerance) {
          uint32list[index] = 0;
        }
      });
      decodeImageFromPixels(
        uint32list.buffer.asUint8List(),
        width,
        height,
        PixelFormat.rgba8888,
        (Image result) => completer.complete(result),
      );
    }
    return completer.future;
  }

  /// Get the real content size of the image.
  Future<Rect> realContentSize({
    Color ignoreColor = const Color(0xffffffff),
    double tolerance = 1,
  }) async {
    Completer<Rect> completer = Completer();
    final ByteData? byteData =
        await toByteData(format: ImageByteFormat.rawRgba);
    if (byteData.isNull) {
      completer.complete(rect);
    } else {
      final double cL = (0.299 * ignoreColor.red +
              0.587 * ignoreColor.green +
              0.114 * ignoreColor.blue) /
          255;
      final double cA = (ignoreColor.green - ignoreColor.red) / 255.0;
      final double cB =
          (ignoreColor.blue - (ignoreColor.green + ignoreColor.red) / 2) /
              255.0;
      final Uint32List uint32list = byteData!.buffer.asUint32List();
      late double minDx = -1;
      late double minDy = -1;
      late double maxDx = -1;
      late double maxDy = -1;
      bool isAssign = false;
      uint32list.iFor((index, element) {
        final Color srcColor = Color(element);
        final double sL = (0.299 * srcColor.red +
                0.587 * srcColor.green +
                0.114 * srcColor.blue) /
            255;
        final double sA = (srcColor.green - srcColor.red) / 255.0;
        final double sB =
            (srcColor.blue - (srcColor.green + srcColor.red) / 2) / 255.0;
        final double dL = cL - sL;
        final double dA = cA - sA;
        final double dB = cB - sB;
        final double dE = sqrt(pow(dL, 2) + pow(dA, 2) + pow(dB, 2));
        if (dE >= tolerance) {
          final int dx = index % width;
          final int dy = index ~/ width;
          if (!isAssign) {
            isAssign = true;
            minDx = dx * 1;
            maxDx = dx * 1;
            minDy = dy * 1;
            maxDy = dy * 1;
          } else {
            minDx = min(dx * 1, minDx);
            minDy = min(dy * 1, minDy);
            maxDx = max(dx * 1, maxDx);
            maxDy = max(dy * 1, maxDy);
          }
        }
      });
      final Offset start = Offset(minDx, minDy);
      final Size realSize = Size((maxDx - minDx), (maxDy - minDy));
      completer.complete(start & realSize);
    }
    return completer.future;
  }

  /// Get the real content in the image.
  Future<Image> realContentImage({
    Color ignoreColor = const Color(0xffffffff),
    bool includeIgnoreColor = false,
    double tolerance = 1,
    bool isDispose = false,
  }) async {
    Completer<Image> completer = Completer();
    final ByteData? byteData =
        await toByteData(format: ImageByteFormat.rawRgba);
    if (byteData.isNull) {
      completer.complete(this);
    } else {
      final double cL = (0.299 * ignoreColor.red +
              0.587 * ignoreColor.green +
              0.114 * ignoreColor.blue) /
          255;
      final double cA = (ignoreColor.green - ignoreColor.red) / 255.0;
      final double cB =
          (ignoreColor.blue - (ignoreColor.green + ignoreColor.red) / 2) /
              255.0;
      final Uint32List uint32list = byteData!.buffer.asUint32List();
      late double minDx = -1;
      late double minDy = -1;
      late double maxDx = -1;
      late double maxDy = -1;
      bool isAssign = false;
      uint32list.iFor((index, element) {
        final Color srcColor = Color(element);
        final double sL = (0.299 * srcColor.red +
                0.587 * srcColor.green +
                0.114 * srcColor.blue) /
            255;
        final double sA = (srcColor.green - srcColor.red) / 255.0;
        final double sB =
            (srcColor.blue - (srcColor.green + srcColor.red) / 2) / 255.0;
        final double dL = cL - sL;
        final double dA = cA - sA;
        final double dB = cB - sB;
        final double dE = sqrt(pow(dL, 2) + pow(dA, 2) + pow(dB, 2));
        if (dE >= tolerance) {
          final int dx = index % width;
          final int dy = index ~/ width;
          if (!isAssign) {
            isAssign = true;
            minDx = dx * 1;
            maxDx = dx * 1;
            minDy = dy * 1;
            maxDy = dy * 1;
          } else {
            minDx = min(dx * 1, minDx);
            minDy = min(dy * 1, minDy);
            maxDx = max(dx * 1, maxDx);
            maxDy = max(dy * 1, maxDy);
          }
        } else if (!includeIgnoreColor) {
          uint32list[index] = 0;
        }
      });
      final Offset start = Offset(minDx, minDy);
      final Size realSize = Size((maxDx - minDx), (maxDy - minDy));
      if (includeIgnoreColor) {
        final PictureRecorder recorder = PictureRecorder();
        final Canvas canvas = Canvas(recorder);
        canvas.drawImageRect(
            this, start & realSize, Offset.zero & realSize, Paint());
        completer.complete(recorder
            .endRecording()
            .toImageSync(realSize.width.toInt(), realSize.height.toInt()));
      } else {
        decodeImageFromPixels(
          uint32list.buffer.asUint8List(),
          width,
          height,
          PixelFormat.rgba8888,
          (Image result) {
            final PictureRecorder recorder = PictureRecorder();
            final Canvas canvas = Canvas(recorder);
            canvas.drawImageRect(
                result, start & realSize, Offset.zero & realSize, Paint());
            completer.complete(recorder
                .endRecording()
                .toImageSync(realSize.width.toInt(), realSize.height.toInt()));
          },
        );
      }
    }
    if (isDispose) dispose();
    return completer.future;
  }
}

==> idkit-0.3.4.tar.gz: lib/src/function/image/func_image.dart <==
import 'dart:async';
import 'dart:math';
import 'dart:typed_data';
import 'dart:ui';
import 'package:flutter/rendering.dart';
import 'package:idkit/idkit.dart';

/// Some Convenience Handling of Fultter Images.
extension IImage on Image {
  /// Image size.
  Size get size => Size(width * 1, height * 1);

  /// Image rectangle with coordinates at (0,0) point.
  Rect rect({Offset offset = Offset.zero}) => offset & size;

  /// Get the Uint8List data of the image.
  Future<Uint8List?> uint8List(
      {ImageByteFormat format = ImageByteFormat.rawRgba}) async {
    final ByteData? byteData = await toByteData(format: format);
    return byteData?.buffer.asUint8List();
  }

  /// Get the Uint8List data of the image.
  Future<Uint32List?> uint32List(
      {ImageByteFormat format = ImageByteFormat.rawRgba}) async {
    final ByteData? byteData = await toByteData(format: format);
    return byteData?.buffer.asUint32List();
  }

  /// Change image size.
  Image changeSizeSync({
    int? imgW,
    int? imgH,
    bool isDispose = false,
  }) {
    final int imgW0 = imgW ?? width;
    final int imgH0 = imgH ?? height;
    final Rect rect = Offset.zero & Size(imgW0 * 1, imgH0 * 1);
    final PictureRecorder pictureRecorder = PictureRecorder();
    final Canvas canvas = Canvas(pictureRecorder, rect);
    canvas.drawImageRect(this, Offset.zero & size, rect, Paint());
    final Picture picture = pictureRecorder.endRecording();
    if (isDispose) dispose();
    return picture.toImageSync(imgW0, imgH0);
  }

  /// Resize an image asynchronously.
  Future<Image> changeSize({
    int? imgW,
    int? imgH,
    bool isDispose = false,
  }) {
    final int imgW0 = imgW ?? width;
    final int imgH0 = imgH ?? height;
    final Rect rect = Offset.zero & Size(imgW0 * 1, imgH0 * 1);
    final PictureRecorder pictureRecorder = PictureRecorder();
    final Canvas canvas = Canvas(pictureRecorder, rect);
    canvas.drawImageRect(this, Offset.zero & size, rect, Paint());
    final Picture picture = pictureRecorder.endRecording();
    if (isDispose) dispose();
    return picture.toImage(imgW0, imgH0);
  }

  /// Image scaling.
  Image scaleSync({
    double scale = 1,
    bool isDispose = false,
  }) {
    final int imgW0 = (width * scale).toInt();
    final int imgH0 = (height * scale).toInt();
    final Rect rect = Offset.zero & Size(imgW0 * 1, imgH0 * 1);
    final PictureRecorder pictureRecorder = PictureRecorder();
    final Canvas canvas = Canvas(pictureRecorder, rect);
    canvas.drawImageRect(this, Offset.zero & size, rect, Paint());
    final Picture picture = pictureRecorder.endRecording();
    if (isDispose) dispose();
    return picture.toImageSync(imgW0, imgH0);
  }

  /// Image scaling.
  Future<Image> scale({
    double scale = 1,
    bool isDispose = false,
  }) {
    final int imgW0 = (width * scale).toInt();
    final int imgH0 = (height * scale).toInt();
    final Rect rect = Offset.zero & Size(imgW0 * 1, imgH0 * 1);
    final PictureRecorder pictureRecorder = PictureRecorder();
    final Canvas canvas = Canvas(pictureRecorder, rect);
    canvas.drawImageRect(this, Offset.zero & size, rect, Paint());
    final Picture picture = pictureRecorder.endRecording();
    if (isDispose) dispose();
    return picture.toImage(imgW0, imgH0);
  }

  /// Image decolorization.
  Future<Image?> removalColor({
    Color color = const Color(0xffffffff),
    double tolerance = 1,
  }) async {
    Completer<Image?> completer = Completer();
    final ByteData? byteData =
        await toByteData(format: ImageByteFormat.rawRgba);
    if (byteData.isNoNull) {
      final double cR = color.red / 255;
      final double cG = color.green / 255;
      final double cB = color.blue / 255;
      final double cL = 0.2126 * cR + 0.7152 * cG + 0.0722 * cB;
      final HSLColor hslColor = HSLColor.fromColor(color);
      final double cH = hslColor.hue / 360;
      final double cS = hslColor.saturation;
      final Uint32List uint32list = byteData!.buffer.asUint32List();
      uint32list.iFor((index, element) {
        final int r = element & 0xff;
        final int g = (element >> 8) & 0xff;
        final int b = (element >> 16) & 0xff;
        final int a = (element >> 24) & 0xff;
        final HSLColor hslColor =
            HSLColor.fromColor(Color.fromARGB(a, r, g, b));
        final double eR = r / 255;
        final double eG = g / 255;
        final double eB = b / 255;
        final double l = 0.2126 * eR + 0.7152 * eG + 0.0722 * eB;
        final double h = hslColor.hue / 360;
        final double s = hslColor.saturation;
        final double dR = cR - eR;
        final double dG = cG - eG;
        final double dB = cB - eB;
        final double dH = cH - h;
        final double dS = cS - s;
        final double dL = cL - l;
        final double dE = sqrt(pow(dR, 2) +
            pow(dG, 2) +
            pow(dB, 2) +
            pow(dH, 2) +
            pow(dL, 2) +
            pow(dS, 2));
        if (dE < tolerance) uint32list[index] = 0;
      });
      decodeImageFromPixels(
        uint32list.buffer.asUint8List(),
        width,
        height,
        PixelFormat.rgba8888,
        (Image result) => completer.complete(result),
      );
    }
    return completer.future;
  }

  /// Get the real content size of the image.
  Future<Rect> realContentSize({
    Color ignoreColor = const Color(0xffffffff),
    double tolerance = 1,
  }) async {
    Completer<Rect> completer = Completer();
    final ByteData? byteData =
        await toByteData(format: ImageByteFormat.rawRgba);
    if (byteData.isNull) {
      completer.complete(rect());
    } else {
      final double cR = ignoreColor.red / 255;
      final double cG = ignoreColor.green / 255;
      final double cB = ignoreColor.blue / 255;
      final double cL = 0.2126 * cR + 0.7152 * cG + 0.0722 * cB;
      final HSLColor hslColor = HSLColor.fromColor(ignoreColor);
      final double cH = hslColor.hue / 360;
      final double cS = hslColor.saturation;
      final Uint32List uint32list = byteData!.buffer.asUint32List();
      late double minDx = -1;
      late double minDy = -1;
      late double maxDx = -1;
      late double maxDy = -1;
      bool isAssign = false;
      uint32list.iFor((index, element) {
        final int r = element & 0xff;
        final int g = (element >> 8) & 0xff;
        final int b = (element >> 16) & 0xff;
        final int a = (element >> 24) & 0xff;
        final HSLColor hslColor =
            HSLColor.fromColor(Color.fromARGB(a, r, g, b));
        final double eR = r / 255;
        final double eG = g / 255;
        final double eB = b / 255;
        final double l = 0.2126 * eR + 0.7152 * eG + 0.0722 * eB;
        final double h = hslColor.hue / 360;
        final double s = hslColor.saturation;
        final double dR = cR - eR;
        final double dG = cG - eG;
        final double dB = cB - eB;
        final double dH = cH - h;
        final double dS = cS - s;
        final double dL = cL - l;
        final double dE = sqrt(pow(dR, 2) +
            pow(dG, 2) +
            pow(dB, 2) +
            pow(dH, 2) +
            pow(dL, 2) +
            pow(dS, 2));
        if (dE >= tolerance) {
          final int dx = index % width;
          final int dy = index ~/ width;
          if (!isAssign) {
            isAssign = true;
            minDx = dx * 1;
            maxDx = dx * 1;
            minDy = dy * 1;
            maxDy = dy * 1;
          } else {
            minDx = min(dx * 1, minDx);
            minDy = min(dy * 1, minDy);
            maxDx = max(dx * 1, maxDx);
            maxDy = max(dy * 1, maxDy);
          }
        }
      });
      final Offset start = Offset(minDx, minDy);
      final Size realSize = Size((maxDx - minDx), (maxDy - minDy));
      completer.complete(start & realSize);
    }
    return completer.future;
  }

  /// Get the real content in the image.
  Future<Image> realContentImage({
    Color ignoreColor = const Color(0xffffffff),
    bool includeIgnoreColor = false,
    double tolerance = 1,
    bool isDispose = false,
  }) async {
    Completer<Image> completer = Completer();
    final ByteData? byteData =
        await toByteData(format: ImageByteFormat.rawRgba);
    if (byteData.isNull) {
      completer.complete(this);
    } else {
      final double cR = ignoreColor.red / 255;
      final double cG = ignoreColor.green / 255;
      final double cB = ignoreColor.blue / 255;
      final double cL = 0.2126 * cR + 0.7152 * cG + 0.0722 * cB;
      final HSLColor hslColor = HSLColor.fromColor(ignoreColor);
      final double cH = hslColor.hue / 360;
      final double cS = hslColor.saturation;
      final Uint32List uint32list = byteData!.buffer.asUint32List();
      late double minDx = -1;
      late double minDy = -1;
      late double maxDx = -1;
      late double maxDy = -1;
      bool isAssign = false;
      uint32list.iFor((index, element) {
        final int r = element & 0xff;
        final int g = (element >> 8) & 0xff;
        final int b = (element >> 16) & 0xff;
        final int a = (element >> 24) & 0xff;
        final HSLColor hslColor =
            HSLColor.fromColor(Color.fromARGB(a, r, g, b));
        final double eR = r / 255;
        final double eG = g / 255;
        final double eB = b / 255;
        final double l = 0.2126 * eR + 0.7152 * eG + 0.0722 * eB;
        final double h = hslColor.hue / 360;
        final double s = hslColor.saturation;
        final double dR = cR - eR;
        final double dG = cG - eG;
        final double dB = cB - eB;
        final double dH = cH - h;
        final double dS = cS - s;
        final double dL = cL - l;
        final double dE = sqrt(pow(dR, 2) +
            pow(dG, 2) +
            pow(dB, 2) +
            pow(dH, 2) +
            pow(dL, 2) +
            pow(dS, 2));
        if (dE >= tolerance) {
          final int dx = index % width;
          final int dy = index ~/ width;
          if (!isAssign) {
            isAssign = true;
            minDx = dx * 1;
            maxDx = dx * 1;
            minDy = dy * 1;
            maxDy = dy * 1;
          } else {
            minDx = min(dx * 1, minDx);
            minDy = min(dy * 1, minDy);
            maxDx = max(dx * 1, maxDx);
            maxDy = max(dy * 1, maxDy);
          }
        } else if (!includeIgnoreColor) {
          uint32list[index] = 0;
        }
      });
      final Offset start = Offset(minDx, minDy);
      final Size realSize = Size((maxDx - minDx), (maxDy - minDy));
      if (includeIgnoreColor) {
        final PictureRecorder recorder = PictureRecorder();
        final Canvas canvas = Canvas(recorder);
        canvas.drawImageRect(
            this, start & realSize, Offset.zero & realSize, Paint());
        completer.complete(recorder
            .endRecording()
            .toImageSync(realSize.width.toInt(), realSize.height.toInt()));
      } else {
        decodeImageFromPixels(
          uint32list.buffer.asUint8List(),
          width,
          height,
          PixelFormat.rgba8888,
          (Image result) {
            final PictureRecorder recorder = PictureRecorder();
            final Canvas canvas = Canvas(recorder);
            canvas.drawImageRect(
                result, start & realSize, Offset.zero & realSize, Paint());
            completer.complete(recorder
                .endRecording()
                .toImageSync(realSize.width.toInt(), realSize.height.toInt()));
          },
        );
      }
    }
    if (isDispose) dispose();
    return completer.future;
  }
}
